import unittest

from src.json_criteria import apply_using_criteria

class TestApplyUsingCriteria(unittest.TestCase):
    def test_apply(self):
        records = [{ 'name': 'Joe' , 'age': 30 }, { 'name': 'John' , 'age': 60 }]
        criteria = { 'AND': [{ 'key': 'age', 'op': 'less_than', 'value': 50 }] }
        def func(record):
            record['is_active'] = True
            return record
        result = apply_using_criteria(records, criteria, func )
        self.assertEqual(result, [{ 'name': 'Joe' , 'age': 30, 'is_active': True }, { 'name': 'John' , 'age': 60 }])

if __name__ == '__main__':
    unittest.main()



from typing import Any
import re

def compare_equal_to(record_value: Any, value: Any) -> bool:
    """Check if the record value is equal to the specified value."""
    return record_value == value

def compare_not_equal_to(record_value: Any, value: Any) -> bool:
    """Check if the record value is not equal to the specified value."""
    return record_value != value

def compare_greater_than(record_value: Any, value: Any) -> bool:
    """Check if the record value is greater than the specified value."""
    return record_value > value

def compare_less_than(record_value: Any, value: Any) -> bool:
    """Check if the record value is less than the specified value."""
    return record_value < value

def compare_greater_than_or_equal_to(record_value: Any, value: Any) -> bool:
    """Check if the record value is greater than or equal to the specified value."""
    return record_value >= value

def compare_less_than_or_equal_to(record_value: Any, value: Any) -> bool:
    """Check if the record value is less than or equal to the specified value."""
    return record_value <= value

def compare_in(record_value: Any, value: Any) -> bool:
    """Check if the record value is in the specified value."""
    return record_value in value

def compare_not_in(record_value: Any, value: Any) -> bool:
    """Check if the record value is not in the specified value."""
    return record_value not in value

def compare_contains(record_value: Any, value: Any) -> bool:
    """Check if the record value contains the specified value."""
    return value in record_value

def compare_not_contains(record_value: Any, value: Any) -> bool:
    """Check if the record value does not contain the specified value."""
    return value not in record_value

def compare_starts_with(record_value: str, value: Any) -> bool:
    """Check if the record value starts with the specified value."""
    return record_value.startswith(value)

def compare_not_starts_with(record_value: str, value: Any) -> bool:
    """Check if the record value does not start with the specified value."""
    return not record_value.startswith(value)

def compare_ends_with(record_value: str, value: Any) -> bool:
    """Check if the record value ends with the specified value."""
    return record_value.endswith(value)

def compare_not_ends_with(record_value: str, value: Any) -> bool:
    """Check if the record value does not end with the specified value."""
    return not record_value.endswith(value)

def compare_matches_regex(record_value: Any, value: Any) -> bool:
    """Check if the record value matches the specified regex value."""
    return re.search(value, str(record_value)) is not None

operator_functions = {
    'equal_to': compare_equal_to,
    'not_equal_to': compare_not_equal_to,
    'greater_than': compare_greater_than,
    'less_than': compare_less_than,
    'greater_than_or_equal_to': compare_greater_than_or_equal_to,
    'less_than_or_equal_to': compare_less_than_or_equal_to,
    'in': compare_in,
    'not_in': compare_not_in,
    'contains': compare_contains,
    'not_contains': compare_not_contains,
    'starts_with': compare_starts_with,
    'not_starts_with': compare_not_starts_with,
    'ends_with': compare_ends_with,
    'not_ends_with': compare_not_ends_with,
    'matches_regex': compare_matches_regex
}

def compare(record_value: Any, operator: str, value: Any) -> bool:
    """
    Compare record value with the specified value using the given operator.

    Parameters:
    - record_value (Any): The value from the data record.
    - operator (str): The comparison operator.
    - value (Any): The value to compare against.

    Returns:
    - bool: The result of the comparison.
    """

    if operator in operator_functions:
        return operator_functions[operator](record_value, value)

    raise ValueError(f"Unsupported operator: {operator}")



import unittest

from src.json_criteria import find_using_criteria

class TestFindUsingCriteria(unittest.TestCase):
    def test_find(self):
        records = [{ 'name': 'Joe' , 'age': 30 }, { 'name': 'Steve', 'age': 60 }]
        criteria = { 'key': 'name', 'op': 'ends_with', 'value': 've' }
        result = find_using_criteria(records, criteria)
        self.assertEqual(result, { 'name': 'Steve', 'age': 60 })

    def test_find_none_found(self):
        records = [{ 'name': 'Joe' , 'age': 30 }, { 'name': 'Steve', 'age': 60 }]
        criteria = { 'key': 'name', 'op': 'in', 'value': ['John'] }
        result = find_using_criteria(records, criteria)
        self.assertEqual(result, None)

if __name__ == '__main__':
    unittest.main()



from .all_meet_criteria import all_meet_criteria
from .any_meet_criteria import any_meet_criteria
from .combine_criteria import combine_criteria
from .filter_using_criteria import filter_using_criteria
from .find_using_criteria import find_using_criteria
from .get_all_criteria import get_all_criteria
from .apply_using_criteria import apply_using_criteria
from .meets_criteria import meets_criteria



from json_criteria import meets_criteria

record = { 'name': 'Joe', 'age': 30 }
criteria = { 'key': 'age', 'op': 'equal_to', 'value': 30 }

# `result` will be True
result = meets_criteria(record, criteria)



from typing import Dict, Any, List
from .internal import meets_crit

def find_using_criteria(
    records: List[Dict[str, Any]],
    criteria: Dict[str, Any]
) -> Dict[str, Any] | None:
    """
    Find the first record that meets the given criteria. Returns `None` if none found.

    Parameters:
    - records (List[Dict[str, Any]]): The data records.
    - criteria (Dict[str, Any]): The criteria. Should be a dictionary representing the conditions that the record must satisfy. May include nested conditions.

    Returns:
    - Dict[str, Any] | None: The first record that meets the criteria, otherwise None.
    """
    return next((record for record in records if meets_crit(record, criteria)), None)



from typing import Dict, Any, List
from .internal import meets_crit

def filter_using_criteria(
    records: List[Dict[str, Any]],
    criteria: Dict[str, Any]
) -> List[Dict[str, Any]]:
    """
    Filter a list of records using the given criteria.

    Parameters:
    - records (List[Dict[str, Any]]): The data records.
    - criteria (Dict[str, Any]): The criteria. Should be a dictionary representing the conditions that the record must satisfy. May include nested conditions.

    Returns:
    - List[Dict[str, Any]]: The filtered records that meet the criteria.
    """
    return [record for record in records if meets_crit(record, criteria)]



import unittest

from src.json_criteria import filter_using_criteria

class TestFilterUsingCriteria(unittest.TestCase):
    def test_filter(self):
        records = [{ 'name': 'Joe' , 'age': 30 }, { 'name': 'Steve', 'age': 60 }]
        criteria = { 'key': 'name', 'op': 'starts_with', 'value': 'S' }
        result = filter_using_criteria(records, criteria)
        self.assertListEqual(result, [{ 'name': 'Steve', 'age': 60 }])

if __name__ == '__main__':
    unittest.main()



from typing import Dict, Any, List
from .internal import meets_crit

def get_all_criteria(
    record: Dict[str, Any],
    criteria_list: List[Dict[str, Any]]
) -> List[Dict[str, Any]]:
    """
    Given a record and a list of criteria, get all criteria that the record meets.

    Parameters:
    - record (Dict[str, Any]): The data record.
    - criteria_list (List[Dict[str, Any]]): The list of criteria to check against.

    Returns:
    - List[Dict[str, Any]]: The list of criteria that the record meets.
    """
    return [criteria for criteria in criteria_list if meets_crit(record, criteria)]



import unittest

from src.json_criteria import get_all_criteria

class TestGetAllCriteria(unittest.TestCase):
    def test_get(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria_list = [
            { 'key': 'name', 'op': 'ends_with', 'value': 've' },
            { 'AND': [{ 'key': 'age', 'op': 'less_than', 'value': 50 }] }
        ]
        result = get_all_criteria(record, criteria_list)
        self.assertEqual(result, [{ 'AND': [{ 'key': 'age', 'op': 'less_than', 'value': 50 }] }])

    def test_get_none(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria_list = [
            { 'key': 'name', 'op': 'ends_with', 'value': 've' },
            { 'AND': [{ 'key': 'age', 'op': 'less_than', 'value': 20 }] }
        ]
        result = get_all_criteria(record, criteria_list)
        self.assertEqual(result, [])

if __name__ == '__main__':
    unittest.main()
